# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot import exceptions
import ubuntuuitoolkit as uitk

from ubuntu_system_tests.helpers.autopilot import order_by_y_coord
from ubuntu_system_tests.helpers.scopes import Scope


class VideoScope(Scope):
    """
    This class represents the preview screen that is part of the
    GenericScopeView and implement the methods specific to the Video Scope.
    """

    def is_in_search_result(self, text):
        """ Indicate if a text is part of the search result as video label """
        try:
            self.scope.select_single(objectName='titleLabel', text=text)
            return True
        except exceptions.StateNotFoundError:
            return False

    def get_my_videos_section(self):
        return self.scope.select_single(
            objectName='dashCategorylocal').select_single('QQuickGridView')

    def get_displayed_thumbnails(self, section=None):
        """
        Retrieve the details for all the displayed thumbnails
        :param section: If not None, the thumbnails will be retrieved from
                        there, otherwise will be retrieved from the scope
        """
        if section:
            labels = section.select_many('Label')
            images = section.select_many('QQuickImage')
        else:
            labels = self.scope.select_many('Label')
            images = self.scope.select_many('QQuickImage')
        label_text = [label.text for label in order_by_y_coord(labels)]
        image_sources = [image.source for image in order_by_y_coord(images)]
        return label_text, image_sources

    def get_my_videos_thumbnails(self, count=None):
        """
        Retrieve all the titles displayed for all the songs in my music
        :param count: The number of expected titles
        """
        my_videos = self.get_my_videos_section()
        if count:
            my_videos.count.wait_for(count)
        return self.get_displayed_thumbnails(section=my_videos)

    def _get_youtube_category(self):
        return self.scope.select_single(
            'DashCategoryBase', category='com.ubuntu.scopes.youtube_youtube')

    def get_youtube_videos(self):
        """ Retrieve all the youtube videos """
        self.wait_for_processing_to_complete()
        category = self._get_youtube_category()
        buttons = category.select_many('AbstractButton')
        return [button for button in buttons if
                'delegate' in button.objectName and button.title]

    def select_youtube_video(self, video):
        """ Select a video in the youtube section """
        self.scope.pointing_device.click_object(video)
        self.wait_for_processing_to_complete()
        return self.scope.select_single(Preview, isCurrent=True)


class Preview(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """
    This class represents the preview Widget that is part of the
    GenericScopeView and implement the methods specific to the Video
    Preview.
    """

    def _get_current(self, objname):
        return self.select_single('PreviewWidget', isCurrentPreview=True,
                                  objectName=objname)

    def get_statistics(self):
        """ Retrieve a string with the likes and dislikes """
        statistics = self._get_current('statistics')
        return statistics.select_single(objectName='titleLabel').text

    def get_summary(self):
        """ Retrieve the summary displayed """
        summary = self._get_current('summary')
        return summary.select_single(objectName='textLabel').text

    def get_title(self):
        return self._get_subheader().title

    def get_subtitle(self):
        return self._get_subheader().subtitle

    def _get_subheader(self):
        """ Retrieve a pair with the title and subtitle displayed """
        header = self._get_current('header')
        return header.select_single(objectName='innerPreviewHeader')

    def _get_video(self):
        return self._get_current('video')

    def play_video(self):
        """
        Click on the play button to start the video playback in the webbrowser
        """
        video = self._get_video()
        play = video.select_single(objectName='playButton')
        self.pointing_device.click_object(play)

    def get_thumbnail(self):
        """ Retrieve the thumbnail url for the current video """
        video = self._get_video()
        thumbnail = video.select_single(objectName='screenshot')
        return thumbnail.source

    def get_external_id(self):
        """ Retrieve the id used to link to the external video """
        video = self._get_video()
        screenshot = video.select_single(objectName='screenshot')
        return screenshot.source.split('/')[-2]
