# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot.matchers import Eventually
from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.tests.base_messaging import MessagingBase
from ubuntu_system_tests.helpers.messaging import Message
from ubuntu_system_tests.helpers import network
from ubuntu_system_tests.helpers.network import(
    fixture_setup as network_fixture
)
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers.indicators import NotificationsIndicatorPage, NotificationsIndicatorItem  # NOQA
from ubuntu_system_tests.helpers.uid import b64_uuid

TIMEOUT_5_MIN = 300


class SMSTestCase(MessagingBase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()

    def test_receive_sms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        message_body = b64_uuid()
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_1.number,
            message_body,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                                          contact=self.contact_1.name))))
        self.assertThat(notification.body, Eventually(Equals(message_body)))
        notification.tap_on()

        self.get_messaging_app()
        self.main_view.get_message(message_body)
        self.assertThat(self.main_view.get_header().title,
                        Eventually(Equals(self.contact_1.name)))

    def test_receive_sms_device_locked(self):
        """
        Receive an SMS from a contact when the device is still locked.
        1. Ensure there are no existing notifications
        2. Receive sms from contact whilst greeter is locked
        3. Check notification is displayed
        4. Swipe down notifications indicator to display new message
        5. Clear all messages

        """
        self.shell.clear_notifications()
        self.assertFalse(self.shell.is_new_notification_displayed())

        message = Message(body=b64_uuid())
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_1.number,
            message.body,
            timeout=TIMEOUT_5_MIN)

        self.assertEqual(notification.summary, 'Message from {name}'.format(
            name=self.contact_1.name
        ))
        self.assertEqual(notification.body, message.body)
        notification.wait_until_destroyed()

        self.assertTrue(self.shell.is_new_notification_displayed())

        messages = self.shell.open_notifications_indicator()
        message = messages.get_message(sender=self.contact_1.name,
                                       body=message.body)
        messages.clear_all()
        message.wait_until_destroyed()
        self.shell.close_indicator_page()
        self.assertFalse(self.shell.is_new_notification_displayed())

    def test_receive_sms_from_unknown_number(self):
        """
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             number and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        message = Message(body=b64_uuid())
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_2.number,
            message.body,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {}'.format(
                                          self.contact_2.number))))
        self.assertThat(notification.body, Eventually(Equals(message.body)))
        notification.tap_on()

        self.get_messaging_app()
        self.main_view.get_message(message.body)
        self.assertThat(self.main_view.get_header().title,
                        Eventually(Equals(self.contact_2.number)))

    def test_receive_sms_with_no_data_connection(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from. Wifi and data connections disabled.
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        # Only turn off data connection as we need wifi to request
        # incoming SMS.
        self.useFixture(network_fixture.TurnOffDataConnection())
        notifications_list = self.shell.get_notifications_list()
        message = Message(body=b64_uuid())
        self.request_incoming_message(self.get_device_phone_number(),
                                      self.contact_1.number,
                                      message.body)
        # Turn off wifi before the incoming SMS
        self.useFixture(network_fixture.TurnOffWifi())
        notification = notifications_list.get_new_message_notification(
            TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                            contact=self.contact_1.name))))
        self.assertThat(notification.body, Eventually(Equals(message.body)))
        notification.tap_on()

        self.get_messaging_app()
        self.main_view.get_message(message.body)
        self.assertThat(self.main_view.get_header().title,
                        Eventually(Equals(self.contact_1.name)))

    @skipUnless(ofono.number_of_sims() > 1, 'More than 1 SIM is required.')
    def test_send_sms_choose_sim(self):
        """
        Prerequisite: 2 SIMs inserted.

        1. Open system settings and set 'Ask me each time' for messages
        2. Open Messaging app.
        3. Choose SIM to send message with
        4. Compose SMS message to contact
        5. Send SMS and verify it is sent using the correct SIM

        """
        self.set_default_sim_for_messages_with_uss('ask')

        self.launch_messaging()
        # choose sim 1 for sending messages
        self.choose_sim_from_prompt('1')
        # send the message and validate it is sent
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    @skipUnless(ofono.number_of_sims() > 1, 'More than 1 SIM is required.')
    def test_send_sms_using_default_sim(self):
        """
        Prerequisite: 2 SIMs inserted.

        1. Open system settings and set 'Ask me each time' for messages
        2. Open Messaging app.
        3. Choose SIM to send message with
        4. Compose SMS message to contact
        5. Send SMS and verify it is sent using the correct SIM

        """
        self.set_default_sim_for_messages_with_uss('1')

        self.launch_messaging()
        # check that no sim selection prompt is displayed
        self.assertFalse(self.main_view.detect_select_sim_prompt())
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_to_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5. Press the Send button
            - The message is added to the conversation and is shown in green
        6. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_to_multiple_contacts(self):
        """
        Prerequisite: Two contacts in the address book with the numbers you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the first contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the name of the second contact to send to in the To.. field
            - The contacts name is stored in the To... field
        5. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        6. Press the Send button
            - The message is added to the conversation and is shown in green
        7. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.create_temporary_contact(self.contact_2)
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1, self.contact_2]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_to_unknown_number(self):
        """
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the number of the contact to send to in the To.. field
            - The contacts number is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5. Press the Send button
            - The message is added to the conversation and is shown in green
        6. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.launch_messaging()
        message = Message(body=b64_uuid())
        self.send_message_to_number_and_validate_reply(self.contact_2, message)

    def turn_on_wifi_and_wait_for_internet(self, timeout):
        """Turn on wifi and ping a server to ensure internet connectivity."""
        network.turn_on_wifi()
        success = network.ensure_internet_connectivity(timeout)
        if not success:
            self.fail("Internet not working after {} seconds".format(timeout))

    def test_send_long_sms_to_contact(self):
        """
        1. Open Messaging app.
        2. Compose long SMS message to contact
        3. Send SMS and verify it is sent

        """
        self.launch_messaging()
        message = Message(body=b64_uuid(length=180))
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_with_no_data_connection(self):
        """
        Prerequisite: A contact in the address book with the number you
                     are sending to.
        1. Open the network indicator by dragging from top bar
            - The network indicator opens with the list of active SIMs
              and wifi status.
        2. Tap the Wifi switch if its already ON
            - Wifi turns off
        3. Tap the 'Cellular settings...' item
            - System settings opens with the cellular page
        4. Tap the cellular data switch if its ON
            - Data connection gets disabled
        5. Open the messaging application
            - The messaging application is opened to the main screen
        6. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        7. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        8. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
        10. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.useFixture(network_fixture.TurnOffInternet())
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message,
                                                           query_twilio=False)
        self.turn_on_wifi_and_wait_for_internet(timeout=30)
        self.validate_outgoing_message_received(recipients, message)
